// main22.cc is a part of the PYTHIA event generator.
// Copyright (C) 2012 Torbjorn Sjostrand.
// PYTHIA is licenced under the GNU GPL version 2, see COPYING for details.
// Please respect the MCnet Guidelines, see GUIDELINES for details.

// Simple illustration how to provide (a) your own resonance-width class,
// and (b) your own cross-section class, with instances handed in to Pythia.
// The hypothetical scenario is that top would have been so long-lived
// that a toponium resonance Theta could form. Then production could
// proceed via q qbar -> gamma*/Z* -> Theta, with decay either to 
// a fermion pair or (dominantly) to three gluons.
// The implementation is not physically correct in any number of ways,
// but should exemplify the strategy needed for realistic cases.

#include "Pythia.h"
#include "SgluonResonance.h"
#include "SigmaSgluonPairs.h"

#include "HepMCInterface.h"
#include "HepMC/GenEvent.h"
#include "HepMC/IO_GenEvent.h"

using namespace Pythia8; 
  
//--------------------------------------------------------------------------

//==========================================================================

//==========================================================================

int main() {

  // Interface for conversion from Pythia8::Event to HepMC one. 
  HepMC::I_Pythia8 ToHepMC;
  //  ToHepMC.set_crash_on_problem();

  // Specify file where HepMC events will be stored.
  HepMC::IO_GenEvent ascii_io("SgluonsHepmc.dat", std::ios::out);
  // Number of events to generate. Max number of errors.
  // Warning: generation of complete events is much slower than if you use
  // PartonLevel:all = off to only get cross sections, so adjust nEvent.
  int nEvent = 1000;
  int nAbort = 5;

  // create the particle (includes PDG_ID)
  ResonanceWidths* sgluonResonance = new SgluonResonance(54711);
  // create the parton process qqbar -> sgl sgl
  SigmaProcess* sigma2qqbar2sgluonsgluonBar = new Sigma2qqbar2sgluonsgluonBar(sgluonResonance->id());
  // create the parton process gg -> sgl sgl
  SigmaProcess* sigma2gg2sgluonsgluonBar = new Sigma2gg2sgluonsgluonBar(sgluonResonance->id());
  
  // Pythia generator.
  Pythia pythia;

  // tell PYTHIA8 about the new particle:
  pythia.setResonancePtr(sgluonResonance);
  // tell PYTHIA8 about the parton processes:
  pythia.setSigmaPtr(sigma2qqbar2sgluonsgluonBar);
  pythia.setSigmaPtr(sigma2gg2sgluonsgluonBar);

  std::stringstream sgluonInput;
  // define the decay header
  sgluonInput << sgluonResonance->id() << ":new = sgluon sgluonBar 1 0 2 500.0 0.2 999. 1001. 0.";
  pythia.readString(sgluonInput.str());
  // Add the sgluon decay channels       WARNING 2 means conditional decay
  sgluonInput.str(std::string());
  sgluonInput << sgluonResonance->id() << ":addChannel = 2 0. 0 21 21";
  pythia.readString(sgluonInput.str());

  // Add the sgluon decay channels    WARNING 3 means conditional decay
  sgluonInput.str(std::string());
  sgluonInput << sgluonResonance->id() << ":addChannel = 3 0. 0 6 -6";
  pythia.readString(sgluonInput.str());

  // Add the sgluon decay channels
  sgluonInput.str(std::string());
  sgluonInput << sgluonResonance->id() << ":addChannel = 0 0. 0 5 -5";
  pythia.readString(sgluonInput.str());

  // Optionally only compare cross sections.
  //  pythia.readString("PartonLevel:all = off");
  pythia.readString("Check:nErrList = 2");
  // center of mass energy
  pythia.readString("Beams:eCM = 8000.");    

  // Initialization for LHC.
  pythia.init();

  // Begin event loop.
  int iAbort = 0;
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {

    // Generate events. Quit if many failures.
    if (!pythia.next()) {
      if (++iAbort < nAbort) continue;
      cout << " Event generation aborted prematurely, owing to error!\n"; 
      break;
    }
    //print the event
    //    pythia.process.list();
    //do a check:
    HepMC::GenEvent* hepmcevt = new HepMC::GenEvent(HepMC::Units::GEV, HepMC::Units::MM);
    // Fill HepMC event, including PDF info.
    ToHepMC.fill_next_event( pythia, hepmcevt );
    // Write the HepMC event to file. Done with it.
    ascii_io << hepmcevt;
    delete hepmcevt;
}

  // Final statistics. Print histogram.
  pythia.stat();
  
  // Done.
  return 0;
}


